==> learntools-0.0.3.tar.gz: learntools-0.0.3/learntools/Learning.py <==
import numpy as np

def random_learning(net,loss_fun,max_its=1000,max_mutations=1000,step=1/2**5,threshold=1e-3,info=False):
    '''
    returns loss over iterations,
    (iterations reached,total mutation attempts)
    '''
    losses = [loss_fun(net)]
    total_k = 0

    for i in range(max_its):
        if info:
            print('Iter',i,'Loss',losses[-1])
        
        if losses[-1] < threshold:
            if info:
                print('Stopped Due to Threshold Reached')
            break
        
        for k in range(max_mutations): #mutate so many times before giving up
            pertubations = random_mutate(net,step)
            loss = loss_fun(net)

            if loss < losses[-1]:
                losses.append(loss)
                break
            else:
                #pertubate in the opposite direction to save generating another pertubation
                undo_mutate(net,pertubations)
                undo_mutate(net,pertubations)

            loss = loss_fun(net)
            if loss < losses[-1]:
                losses.append(loss)
                break
            else:
                make_mutate(net,pertubations)
            
        total_k += k
        if k == max_mutations-1:
            if info:
                print('Stopped Due to Max Mutations Reached')
            break

    return losses,(i,total_k)

def random_mutate(net,step): #randomly mutate the network
    pertubations = []
    for index in net.mutateable_layers:
        w_shape = np.shape(net.layers[index].weights)
        b_shape = np.shape(net.layers[index].biases)

        w_pertubation = step * np.random.normal(size = w_shape)
        b_pertubation = step * np.random.normal(size = b_shape)

        pertubations.append([w_pertubation,b_pertubation])

        net.layers[index].weights += w_pertubation
        net.layers[index].biases += b_pertubation

    return pertubations

def undo_mutate(net,pertubations):
    i = 0
    for index in net.mutateable_layers:
        w_pertubation = pertubations[i][0]
        b_pertubation = pertubations[i][1]

        net.layers[index].weights -= w_pertubation
        net.layers[index].biases -= b_pertubation

        i += 1

def make_mutate(net,pertubations):
    i = 0
    for index in net.mutateable_layers:
        w_pertubation = pertubations[i][0]
        b_pertubation = pertubations[i][1]

        net.layers[index].weights += w_pertubation
        net.layers[index].biases += b_pertubation

        i += 1

def random_evolution_learning(net,loss_fun,max_its=1000,max_mutations=1000,step=1/2**5,threshold=1e-3,info=False):
    return 0

def test(n):
    return n+1

==> learntools-0.0.8.tar.gz: learntools-0.0.8/learntools/Learning.py <==
import numpy as np
from copy import deepcopy

def random_learning(net,loss_fun,max_its=1000,max_mutations=1000,step=1/2**5,threshold=1e-3,info=False):
    '''
    returns loss over iterations,
    (iterations reached,total mutation attempts)
    '''
    losses = [loss_fun(net)]
    total_k = 0

    for i in range(max_its):
        if info:
            print('Iter',i,'Loss',losses[-1])
        
        if losses[-1] < threshold:
            if info:
                print('Stopped Due to Threshold Reached')
            break
        
        for k in range(max_mutations): #mutate so many times before giving up
            pertubations = random_mutate(net,step)
            loss = loss_fun(net)

            if loss < losses[-1]:
                losses.append(loss)
                break
            else:
                #pertubate in the opposite direction to save generating another pertubation
                undo_mutate(net,pertubations)
                undo_mutate(net,pertubations)

            loss = loss_fun(net)
            if loss < losses[-1]:
                losses.append(loss)
                break
            else:
                make_mutate(net,pertubations)
            
        total_k += k
        if k == max_mutations-1:
            if info:
                print('Stopped Due to Max Mutations Reached')
            break

    return losses,(i,total_k)

def random_mutate(net,step: float): #randomly mutate the network
    pertubations = []
    for index in net.mutateable_layers:
        w_shape = np.shape(net.layers[index].weights)
        b_shape = np.shape(net.layers[index].biases)

        w_pertubation = step * np.random.normal(size = w_shape)
        b_pertubation = step * np.random.normal(size = b_shape)

        pertubations.append([w_pertubation,b_pertubation])

        net.layers[index].weights += w_pertubation
        net.layers[index].biases += b_pertubation

    return pertubations

def undo_mutate(net,pertubations):
    i = 0
    for index in net.mutateable_layers:
        w_pertubation = pertubations[i][0]
        b_pertubation = pertubations[i][1]

        net.layers[index].weights -= w_pertubation
        net.layers[index].biases -= b_pertubation

        i += 1

def make_mutate(net,pertubations):
    i = 0
    for index in net.mutateable_layers:
        w_pertubation = pertubations[i][0]
        b_pertubation = pertubations[i][1]

        net.layers[index].weights += w_pertubation
        net.layers[index].biases += b_pertubation

        i += 1

def random_evolution_learning(base_net,loss_fun,population = 100,step = 2**-5,threshold=1e-3):
    '''
    generates a population of neural networks that will progress 
    via asexual reproduction
    '''
    pop = [deepcopy(base_net) for i in range(population)]



    return 0
